# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['multiwidth']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'multiwidth',
    'version': '1.0.1',
    'description': 'A package for reading and writing mixed width tables',
    'long_description': '# Mixed Width\n\n## About\n\nThis project is designed to enable the easy reading/writing of fixed-width files with variable widths. For example:\n\n```\nFOO     BAR     BAZ\nHello   World   SomeExtraLongString\n```\n\nIn this case we have differing widths between the columns. This means we can\'t just specify a single width for the columns. This might not be much of an issue itself, just code in the differing widths and parse using that, however with things like console output we can\'t rely on a consistent width for cells, which is where Mixed Width comes in. To resolve this, it detects the start of cells by looking for location where spaces precede letters which line up across all lines of the files. Using this method we are able to parse files with unknown widths and convert them into either a list of lists, or a list of dictionaries where the keys are the header values.\n\n## Examples\n\nTo use `multiwidth` for the most part it\'s the same as the built-in `json` module. This means that parsing some output takes the form of:\n\n```python\nimport multiwidth\n\nstring_to_parse = """FOO     BAR     BAZ\nHello   World   SomeExtraLongString"""\n\ndata = multiwidth.loads(string_to_parse)\n\nprint(data)\n# output:\n# [[\'Hello\', \'World\', \'SomeExtraLongString\']]\n```\n\nIf preserving the headers is important, `output_json=True` can be added to the `loads` method:\n\n```python\nimport multiwidth\n\nstring_to_parse = """FOO     BAR     BAZ\nHello   World   SomeExtraLongString"""\n\ndata = multiwidth.loads(string_to_parse, output_json=True)\n\nprint(data)\n# output:\n# [{\'FOO\': \'Hello\', \'BAR\': \'World\', \'BAZ\': \'SomeExtraLongString\'}]\n```\n\nEach line will then be a dictionary with the header keys and their corresponding values\n\nIn addition, if the content is stored in a file, `multiwidth.load(<file_object>)` can be used. \n\nFinally, data can be output as well from multiwidth. \n\n```python\nimport multiwidth\n\nheaders = [\'FOO\', \'BAR\', \'BAZ\']\ndata = [[\'Hello\', \'World\', \'SomeExtraLongString\']]\n\nprint(multiwidth.dumps(data, headers=headers))\n\n# Output:\n# FOO   BAR   BAZ\n# Hello World SomeExtraLongString \n```\n\nYou can also control the spacing between columns with `cell_suffix=\'<your desired padding between columns>\'`. For example:\n\n```python\nimport multiwidth\n\nheaders = [\'FOO\', \'BAR\', \'BAZ\']\ndata = [[\'Hello\', \'World\', \'SomeExtraLongString\']]\n\nprint(multiwidth.dumps(data, headers=headers, cell_suffix=\'   \'))\n\n# Output:\n# FOO     BAR     BAZ\n# Hello   World   SomeExtraLongString \n```\n\nYou can also dump JSON data by omitting the `headers` argument:\n\n```python\nimport multiwidth\n\ndata = [{\'FOO\': \'Hello\', \'BAR\': \'World\', \'BAZ\': \'SomeExtraLongString\'}]\n\nprint(multiwidth.dumps(data))\n\n# Output:\n# FOO   BAR   BAZ\n# Hello World SomeExtraLongString\n```\n\nFinally, you can dump to a file with `dumps(<your file object>)`\n\n## Usage\n\n**load**\n\n```python\n"""Parse data from a file object\n\nArgs:\n    file_object (io.TextIOWrapper): File object to read from\n    padding (str, optional): Which character takes up the space to create the fixed\n        width. Defaults to " ".\n    header (bool, optional): Does the file contain a header. Defaults to True.\n    output_json (bool, optional): Should a list of dictionaries be returned instead\n        of a list of lists. Defaults to False. Requires that \'header\' be set to\n        True.\n\nReturns:\n    Union[List[List],List[Dict]]: Either a list of lists or a list of dictionaries that\n        represent the extracted data\n"""\n```\n\n**loads**\n\n```python\n"""Takes a string of a fixed-width file and breaks it apart into the data contained.\n\nArgs:\n    contents (str): String fixed-width contents.\n    padding (str, optional): Which character takes up the space to create the fixed\n        width. Defaults to " ".\n    header (bool, optional): Does the file contain a header. Defaults to True.\n    output_json (bool, optional): Should a list of dictionaries be returned instead\n        of a list of lists. Defaults to False. Requires that \'header\' be set to\n        True.\n\nRaises:\n    Exception: \'output_json\' is True but \'header\' is False.\n\nReturns:\n    List[List] | List[Dict]: Either a list of lists or a list of dictionaries that\n        represent the extracted data\n"""\n```\n\n**dump**\n\n```python\n"""Dumps a formatted table to a file\n\nArgs:\n    data (Union[List[List],List[Dict]]): Data to dump to a file. If using JSON data\n        then omit the `headers` argument\n    file_object (io.TextIOWrapper): File object to write to\n    headers (List[str], optional): Headers to use with list data. Defaults to None.\n    padding (str, optional): Character to use as padding between values. Defaults to\n        \' \'.\n    cell_suffix (str, optional): String to use as the padding between columns.\n        Defaults to \' \'.\n"""\n```\n\n**dumps**\n\n```python\n"""Dumps a formatted table to a string\n\nArgs:\n    data (Union[List[List],List[Dict]]): List or dictionary data to format\n    headers (List[str], optional): Headers to use with list data. Defaults to None.\n    padding (str, optional): Character to use as padding between values. Defaults to\n        \' \'.\n    cell_suffix (str, optional): String to use as the padding between columns.\n        Defaults to \' \'.\n\nReturns:\n    str: Formatted table of input data\n"""\n```\n\n## License\n\nMultiwidth is under the [MIT license](https://opensource.org/licenses/MIT).\n\n## Contact\n\nIf you have any questions or concerns please reach out to me (John Carter) at [jfcarter2358@gmail.com](mailto:jfcarter2358@gmail.com)\n',
    'author': 'John Carter',
    'author_email': 'jfcarter2358@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/jfcarter2358/multiwidth',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
